@@ -237,6 +237,9 @@ void open_ao_playdevice(struct mad_header const *header)
         format.bits = 16;
         format.rate = header->samplerate;
         format.channels = (options.opt & MPG321_FORCE_STEREO) ? 2 : MAD_NCHANNELS(header);
+	
+	/* Add this element as an option to mpg321 */
+	format.matrix = "L,R";
 
         /* mad gives us little-endian data; we swap it on big-endian targets, to
           big-endian format, because that's what most drivers expect. */
