@@ -1,6 +1,7 @@
 /*
     mpg321 - a fully free clone of mpg123.
     Copyright (C) 2001 Joe Drew
+    Copyright (C) 2006-2010 Nanakos Chrysostomos
     
     Originally based heavily upon:
     plaympeg - Sample MPEG player using the SMPEG library
@@ -49,6 +50,7 @@
 
 #include "getopt.h" /* GNU getopt is needed, so I included it */
 #include "mpg321.h"
+#include <sys/wait.h>
 
 #include <id3tag.h>
 
@@ -63,8 +65,8 @@ struct termios tty_ts, tty_ts_orig; /* termios*/
 struct termios *tty_ts_orig_pt = NULL;
 static char temp[1024];
 char *ctty_path();
-
-
+int set_xterm = 0;
+char title[BUFSIZ];
 
 int shuffle_play;
 int stop_playing_file = 0;
@@ -78,6 +80,52 @@ int file_change = 0;
 
 char *id3_get_tag (struct id3_tag const *tag, char const *what, unsigned int maxlen);
 
+/* Ignore child processes from the AudioScrobbler helper */
+RETSIGTYPE handle_sigchld(int sig)
+{
+	int stat;
+	while(waitpid(-1, &stat, WNOHANG) > 0)
+		;
+}
+
+static struct {
+	int index;
+	const char *id;
+	const char *name;
+} const info_id3[] = {
+	{ 0 ,	ID3_FRAME_TITLE,	"Title	: "	},
+	{ 1 ,	ID3_FRAME_ARTIST,	" Artist : "	},
+	{ 2 ,	ID3_FRAME_ALBUM,	"Album	: "	},
+	{ 3 ,	ID3_FRAME_YEAR,		" Year	 : "	},
+	{ 4 ,	ID3_FRAME_COMMENT,	"Comment : "	},
+	{ 5 ,	ID3_FRAME_GENRE,	" Genre : "	}
+};
+
+/* Parse an ID3 tag into TITLE, ARTIST, ALBUM, YEAR, COMMENT, GENRE
+ * and return true if at least one of those is present.
+ */
+
+static int parse_id3( char *names[], const struct id3_tag *tag)
+{
+	int found, i;
+
+	found = 0;
+	/* Get ID3 tag if available, 30 chars except 4 for year */
+	for(i=0; i<=5; i++)
+	{
+		names[i] = NULL;
+		names[i] = id3_get_tag(tag, info_id3[i].id, (i==3) ? 4 : 30);
+		if( names[i] != NULL && names[i][0] != '\0' )
+			found = 1;
+	}
+
+	return (found);
+}
+
+
+
+
+
 void mpg123_boilerplate()
 {
     fprintf(stderr,"High Performance MPEG 1.0/2.0/2.5 Audio Player for Layer 1, 2, and 3.\n"
@@ -119,6 +167,10 @@ void usage(char *argv0)
         "   --loop N or -l N         Play files N times. 0 means until\n"
         "                            interrupted\n"
         "   -R                       Use remote control interface\n"
+        "   -F                       Turn on FFT analysis on PCM data. Remote mode only\n"
+        "   -B                       Read recursively the given directories\n"
+        "   -S                       Report mp3 file to AudioScrobbler\n"
+        "   -x                       Set xterm title setting\n"
         "   -p hostname:port         Use proxy server\n"
         "   -u username:password     Use proxy server basic authentication\n"
         "   -U username:password     Use proxy server basic authentication by using environment variables\n"
@@ -168,36 +220,13 @@ RETSIGTYPE handle_signals(int sig)
 static int show_id3(struct id3_tag const *tag)
 {
     unsigned int i;
-    int print = 0;
     char emptystring[31];
     char *names[6];
-    struct {
-        int index;
-        char const *id;
-        char const *name;
-    } const info[] = {
-        { 0,    ID3_FRAME_TITLE,  "Title  : "   },
-        { 1,    ID3_FRAME_ARTIST, "  Artist: "  },
-        { 2,    ID3_FRAME_ALBUM,  "Album  : "   },
-        { 3,    ID3_FRAME_YEAR,   "  Year  : "  },
-        { 4,    ID3_FRAME_COMMENT,"Comment: "   },
-        { 5,    ID3_FRAME_GENRE,  "  Genre : "  }
-    };
 
     memset (emptystring, ' ', 30);
     emptystring[30] = '\0';
-    /*  Get ID3 tag if available, 30 chars except 4 for year  */
-    for (i=0; i<=5; i++)    {
-        names[i] = NULL;
-        names[i] = id3_get_tag(tag, info[i].id, (i==3) ? 4 : 30);
-    }
-    for (i=0; i<=5; i++)    {
-        if (names[i])   {
-            print = 1;
-            break;
-        }
-    }
-    if (!print) {
+    
+    if (!parse_id3(names, tag)) {
         return 0;
     }
 
@@ -225,7 +254,7 @@ static int show_id3(struct id3_tag const *tag)
     {
         /* Emulate mpg123 original behaviour  */
         for (i=0; i<=5; i++)    {
-            fprintf (stderr, "%s", info[i].name);
+            fprintf (stderr, "%s", info_id3[i].name);
             if (!names[i])  {
                 fprintf (stderr, emptystring);
             }   else    {
@@ -239,6 +268,28 @@ static int show_id3(struct id3_tag const *tag)
     return 1;
 }
 
+static int get_id3_info( const char *fname, struct id3_file **id3struct, struct id3_tag **id3tag)
+{
+	struct id3_file *s;
+	struct id3_tag *t;
+
+	s = id3_file_open(fname, ID3_FILE_MODE_READONLY);
+	if( s == NULL )
+		return (0);
+
+	t = id3_file_tag(s);
+	if(t == NULL)
+	{
+		id3_file_close(s);
+		return (0);
+	}
+
+	*id3struct = s;
+	*id3tag = t;
+	return (1);
+}
+
+
 int main(int argc, char *argv[])
 {
     int fd = 0;
@@ -270,6 +321,15 @@ int main(int argc, char *argv[])
     /* Get the command line options */
     parse_options(argc, argv, pl);
 
+    if(options.opt & MPG321_PRINT_FFT)
+	    if(!(options.opt & MPG321_REMOTE_PLAY))
+	    {
+		    /* printf("FFT analysis can only be used in Remote mode play.\n\n"); */
+		    usage(argv[0]);			
+		    exit(0);
+	    }
+
+
     /* If there were no files and no playlist specified just print the usage */
     if (!playlist_file && optind == argc)
     {
@@ -279,8 +339,11 @@ int main(int argc, char *argv[])
 
     if (playlist_file)
         load_playlist(pl, playlist_file);
-    
-    add_cmdline_files(pl, argv);
+
+    if(options.opt & MPG321_RECURSIVE_DIR)
+	    add_cmdline_files_recursive_dir(pl, argv);
+    else
+	    add_cmdline_files(pl, argv);
 
     if (shuffle_play)
         shuffle_files(pl);
@@ -303,7 +366,11 @@ int main(int argc, char *argv[])
     {
         printf ("@R MPG123\n");
     }
-    tty_control();    
+    if(set_xterm)
+    {
+	    tty_control();
+	    get_term_title(title);
+    }
     /* Play the mpeg files or zip it! */
     while((currentfile = get_next_file(pl, &playbuf)))
     {
@@ -330,32 +397,54 @@ int main(int argc, char *argv[])
         
         mad_timer_reset(&current_time);
 
+	id3struct = NULL;
+
         if (!(options.opt & MPG321_QUIET_PLAY) && file_change)
         {
-            id3struct = id3_file_open (currentfile, ID3_FILE_MODE_READONLY);
+           /* id3struct = id3_file_open (currentfile, ID3_FILE_MODE_READONLY);*/
 
-            if (id3struct)
-            {
-                id3tag = id3_file_tag (id3struct);
-            
-                if (id3tag)
-                {
-                    show_id3 (id3tag);
-                }
+            if (id3struct == NULL)
+		    get_id3_info(currentfile, &id3struct, &id3tag);
+	    if(id3tag)
+		    show_id3(id3tag);
+	}
 
-                id3_file_close (id3struct);
-            }
-        }
+	scrobbler_time = -1;
+	if(options.opt & MPG321_USE_SCROBBLER)
+	{
+		if(id3struct == NULL)
+			get_id3_info(currentfile,&id3struct,&id3tag);
+                
+	    if (id3tag)
+	    {
+		    char emptystring[31], emptyyear[5] = "    ";
+		    int i;
+
+		    if(parse_id3(scrobbler_args, id3tag))
+		    {
+			    memset(emptystring, ' ', 30);
+			    emptystring[30] = '\0';
+	    		    if((options.opt & MPG321_VERBOSE_PLAY) && (options.opt & MPG321_USE_SCROBBLER))
+			    {
+				    fprintf(stderr, "\nPreparing for the AudioScrobbler:\n");
+				    for(i = 0; i < 6; i++)
+				    {
+					    if(scrobbler_args[i] == NULL)
+						    scrobbler_args[i] =
+							    ( i == 3 ? emptyyear: emptystring);
+					    fprintf(stderr, "- %s\n", scrobbler_args[i]);
+				    }
+			    }
+		    }
+	    }
+	}
+      
 
         if (options.opt & MPG321_REMOTE_PLAY && file_change)
         {
-            id3struct = id3_file_open (currentfile, ID3_FILE_MODE_READONLY);
-
-            if (id3struct)
-            {
-                id3tag = id3_file_tag (id3struct);
-            
-                if (id3tag)
+		if(id3struct == NULL)
+			get_id3_info(currentfile, &id3struct, &id3tag);
+		if(id3tag)
                 {
                     if (!show_id3(id3tag))
                     {
@@ -376,14 +465,6 @@ int main(int argc, char *argv[])
                     }
                 }
                 
-                else
-                {
-                    fprintf(stderr, "Allocation error");
-                    exit(1);
-                }
-
-                id3_file_close (id3struct);
-            }
             
             else
             {
@@ -401,6 +482,9 @@ int main(int argc, char *argv[])
             }
         }
 
+	if(id3struct != NULL)
+		id3_file_close(id3struct);
+
         /* Create the MPEG stream */
         /* Check if source is on the network */
         if((fd = raw_open(currentfile)) != 0 || (fd = http_open(currentfile)) != 0
@@ -411,7 +495,7 @@ int main(int argc, char *argv[])
             playbuf.length = BUF_SIZE;
             
             mad_decoder_init(&decoder, &playbuf, read_from_fd, read_header, /*filter*/0,
-                            output, /*error*/0, /* message */ 0);
+                            output, handle_error, /* message */ 0);
         }
 
         /* Check if we are to use stdin for input */
@@ -422,7 +506,7 @@ int main(int argc, char *argv[])
             playbuf.length = BUF_SIZE;
 
             mad_decoder_init(&decoder, &playbuf, read_from_fd, read_header, /*filter*/0,
-                            output, /*error*/0, /* message */ 0);
+                            output, handle_error, /* message */ 0);
         }
             
         /* currentfile is a local file (presumably.) mmap() it */
@@ -433,8 +517,9 @@ int main(int argc, char *argv[])
             if((fd = open(currentfile, O_RDONLY)) == -1)
             {
                 mpg321_error(currentfile);
-
+		exit(1);
                 /* mpg123 stops immediately if it can't open a file */
+		/* If sth goes wrong break!!!*/
                 break;
             }
             
@@ -451,6 +536,12 @@ int main(int argc, char *argv[])
             
             calc_length(currentfile, &playbuf);
 
+	    if((options.opt & MPG321_VERBOSE_PLAY) && (options.opt & MPG321_USE_SCROBBLER))
+		    fprintf(stderr, "Track duration: %ld seconds\n",playbuf.duration.seconds);
+
+	    if(options.opt & MPG321_USE_SCROBBLER)
+		    scrobbler_set_time(playbuf.duration.seconds);
+
             if ((options.maxframes != -1) && (options.maxframes <= playbuf.num_frames))
             { 
                 playbuf.max_frames = options.maxframes;
@@ -469,7 +560,7 @@ int main(int argc, char *argv[])
             playbuf.frames[0] = playbuf.buf;
             
             mad_decoder_init(&decoder, &playbuf, read_from_mmap, read_header, /*filter*/0,
-                            output, /*error*/0, /* message */ 0);
+                            output, handle_error, /* message */ 0);
         }
 
         if(!(options.opt & MPG321_QUIET_PLAY))/*zip it!!!*/
@@ -503,15 +594,22 @@ int main(int argc, char *argv[])
             fprintf(stderr,"Playing MPEG stream from %s ...\n", basen);
             
 			/*Printing xterm title*/
-			osc_print(0,0,basen);
+	    if(set_xterm)
+	    {
+		    osc_print(0,0,basen);
+	    }
 	    
             free(dirc);
             free(basec);
         }    
 
         signal(SIGINT, handle_signals);
+	signal(SIGCHLD, handle_sigchld);
         /*Give control back so that we can implement SIG's*/
-        set_tty_restore();
+	if(set_xterm)
+	{
+		set_tty_restore();
+	}
         /* Every time the user gets us to rewind, we exit decoding,
            reinitialize it, and re-start it */
       
@@ -570,12 +668,13 @@ int main(int argc, char *argv[])
 
     ao_shutdown();
     /*Restoring TTY*/
-    set_tty_restore();
-    osc_print(0,0,"Terminal");
-    if (ctty)
-    fclose(ctty);
-
-
+    if(set_xterm)
+    {
+	    set_tty_restore();
+	    osc_print(0,0,title);
+	    if (ctty)
+		    fclose(ctty);
+    }
     return(0);
 }
 
@@ -804,3 +903,69 @@ char *ctty_path()
                 break;
    return tty_path;
 }
+
+int tty_read(char *output,size_t size)
+{
+	int n;
+
+	n = read(TTY_FILENO,output,size-1);
+	if(n == -1)
+	{
+		perror("read");
+	}
+	 return n;
+}
+
+void get_term_title(char *title)
+{
+	int n;
+	static char buffer[BUFSIZ];
+	char temp[20];
+
+	snprintf(temp,sizeof(temp),"\033[%dt",21);
+
+	raw_print(temp);
+
+	n = tty_read(buffer,sizeof(buffer));
+	if(n==1)
+		n+= tty_read(buffer+1,sizeof(buffer)-1);
+
+	while( !(buffer[n-2] == '\033' && buffer[n-1] == '\\') ) {
+		n += tty_read(buffer+n,sizeof(buffer)-n);
+	}
+
+	buffer[n-2]= '\0';
+
+	snprintf((char *)title,sizeof(buffer),"%s",buffer+3);
+}
+
+static enum mad_flow handle_error(void *data, struct mad_stream *stream, struct mad_frame *frame)
+{
+	signed long tagsize;
+
+	switch(stream->error)
+	{
+		case MAD_ERROR_BADDATAPTR:
+			return MAD_FLOW_CONTINUE;
+		case MAD_ERROR_LOSTSYNC:
+			tagsize = id3_tag_query(stream->this_frame,stream->bufend - stream->this_frame);
+			if(tagsize > 0)
+			{
+				mad_stream_skip(stream, tagsize);
+				return MAD_FLOW_CONTINUE;
+			}
+
+		default:
+			break;
+	}
+
+	if(stream->error == MAD_ERROR_BADCRC)
+	{
+		mad_frame_mute(frame);
+		return MAD_FLOW_IGNORE;
+	}
+
+	return MAD_FLOW_CONTINUE;
+}
+
+
