@@ -1,6 +1,7 @@
 /*
     mpg321 - a fully free clone of mpg123.
     Copyright (C) 2001 Joe Drew
+    Copyright (C) 2006, 2007, 2008, 2009, 2010 Nanakos Chrysostomos
     
     Originally based heavily upon:
     plaympeg - Sample MPEG player using the SMPEG library
@@ -38,8 +39,12 @@
 #include <ao/ao.h>
 #include <mad.h>
 
-#define FAKEVERSION "0.2.11"
-#define VERSIONDATE "2009/06/16"
+#define FAKEVERSION "0.2.12-1"
+#define VERSIONDATE "2010/07/01"
+
+#ifndef PATH_MAX
+#define PATH_MAX	4096
+#endif
 
 /* playlist structure */
 typedef struct pl
@@ -112,6 +117,8 @@ extern int file_change;
 int loop_remaining;
 
 extern int status;
+extern int scrobbler_time;
+extern char *scrobbler_args[6];
 
 enum
 {
@@ -124,24 +131,27 @@ enum
 
 enum
 {
-    MPG321_VERBOSE_PLAY  = 0x00000001,
-    MPG321_QUIET_PLAY    = 0x00000002,
-    MPG321_REMOTE_PLAY   = 0x00000004,
+    MPG321_VERBOSE_PLAY   = 0x00000001,
+    MPG321_QUIET_PLAY     = 0x00000002,
+    MPG321_REMOTE_PLAY    = 0x00000004,
 
-    MPG321_USE_OSS       = 0x00000010,
-    MPG321_USE_SUN       = 0x00000020,
-    MPG321_USE_ALSA      = 0x00000040,
-    MPG321_USE_ESD       = 0x00000080,
-    MPG321_USE_ARTS      = 0x00000100,
-    MPG321_USE_NULL      = 0x00000200,
-    MPG321_USE_STDOUT    = 0x00000400,
-    MPG321_USE_WAV       = 0x00000800,
-    MPG321_USE_AU        = 0x00001000,
-    MPG321_USE_CDR       = 0x00002000,
-    MPG321_USE_USERDEF   = 0x00004000,
-    MPG321_USE_ALSA09    = 0x00008000,
+    MPG321_USE_OSS        = 0x00000010,
+    MPG321_USE_SUN        = 0x00000020,
+    MPG321_USE_ALSA       = 0x00000040,
+    MPG321_USE_ESD        = 0x00000080,
+    MPG321_USE_ARTS       = 0x00000100,
+    MPG321_USE_NULL       = 0x00000200,
+    MPG321_USE_STDOUT     = 0x00000400,
+    MPG321_USE_WAV        = 0x00000800,
+    MPG321_USE_AU         = 0x00001000,
+    MPG321_USE_CDR        = 0x00002000,
+    MPG321_USE_USERDEF    = 0x00004000,
+    MPG321_USE_ALSA09     = 0x00008000,
     
-    MPG321_FORCE_STEREO  = 0x00010000
+    MPG321_FORCE_STEREO   = 0x00010000,
+    MPG321_USE_SCROBBLER  = 0x00020000,
+    MPG321_RECURSIVE_DIR  = 0x00040000,
+    MPG321_PRINT_FFT	  = 0x00080000,
 };
 
 #define DEFAULT_PLAYLIST_SIZE 1024
@@ -152,6 +162,7 @@ playlist * new_playlist();
 void resize_playlist(playlist *pl);
 char * get_next_file(playlist *pl, buffer *buf);
 void add_cmdline_files(playlist *pl, char *argv[]);
+void add_cmdline_files_recursive_dir(playlist *pl, char *argv[]);
 void add_file(playlist *pl, char *file);
 void load_playlist(playlist *pl, char *filename);
 void set_random_play(playlist *pl);
@@ -174,6 +185,8 @@ enum mad_flow read_header(void *data, struct mad_header const * header);
 enum mad_flow output(void *data, struct mad_header const *header, struct mad_pcm *pcm);
 int calc_length(char *file, buffer*buf );
 
+static enum mad_flow handle_error(void *data, struct mad_stream *stream, struct mad_frame *frame);
+
 enum mad_flow move(buffer *buf, signed long frames);
 void seek(buffer *buf, signed long frame);
 void pause_play(buffer *buf, playlist *pl);
@@ -203,6 +216,30 @@ int set_tty_restore();
 int raw_print(char *ctlseq);
 int osc_print(int ps1,int ps2,char *pt);
 char *ctty_path();
+void get_term_title(char *title);
+int tty_read(char *output,size_t size);
 
+/* AudioScrobbler functions */
+void scrobbler_report(void);
+void scrobbler_set_time(long);
+void scrobbler_set_verbose(int);
+RETSIGTYPE handle_sigchld(int sig);
+
+/* FFT data structures */
+#define FFT_BUFFER_SIZE_LOG 9
+#define FFT_BUFFER_SIZE (1 << FFT_BUFFER_SIZE_LOG) /* 512 */
+/*Temporary data stores to perform FFT in */
+double real[FFT_BUFFER_SIZE];
+double imag[FFT_BUFFER_SIZE];
+
+typedef struct {
+	double real[FFT_BUFFER_SIZE];
+	double imag[FFT_BUFFER_SIZE];
+} fft_state;
+
+typedef short int sound_sample;
+//void fft_perform(const sound_sample *input, double *output, fft_state *state);
+
+fft_state *fft_init(void);
 
 #endif /* _MPG321_H_ */
